/*
 Name:		UrsUsart1Dma
 Created:	18.10.2021
 Author:	   Ulli

 Version 1.0 (2021-10-18)
 -------------------------
 - Basis-Version

Version 1.1 (2021-10-22)
 -------------------------
 - Fr die Initialisierung der GPIOs wird die libmaple-Funktion gpio_set_mode benutzt.
*/

#ifndef _URSUSART1DMA_h
#define _URSUSART1DMA_h

#include <Arduino.h>
#include <libmaple/dma.h>

// Klasse zum Versenden von Byte-Arrays ber die USART1 per DMA
class UrsUsart1DmaClass {

protected:
   static dma_tube_reg_map* channel; // Der zur USART1-TX gehrende DMA-Kanal (DMA1 CH4)

   static volatile bool busy;   // true, solange der Sendevorgang luft.

   static void initDMA();       // belegt die DMA-Register
   static void initUSART1(uint32 baud); // belegt die USART1-Register

   static void DMA1_Channel4_IRQHandler(void); // ISR fr Transfer komplett

   // Keine freie Instanziierung
   UrsUsart1DmaClass() {}
   UrsUsart1DmaClass(UrsUsart1DmaClass const&) = delete;
   void operator=(UrsUsart1DmaClass const&) = delete;

public:
   // Singleton fr diese Klasse
   static UrsUsart1DmaClass& getInstance();

   // Initialisiert die USART1 mit der angegebenen Baudrate (max. 4,5 Mbaud)
   static void begin(uint32_t baud);

   // Versendet size Bytes aus der Quelle source
   inline static void xmit(const uint8_t* source, uint16_t size) {
      while (isBusy()); // Auf Beendigung einer laufenden bertragung warten.
      busy = true;

      channel->CNDTR = size; // Set the data size in CNDTR Register
      channel->CMAR = (uint32_t)source; // Set the  Memory address in MAR Register
      channel->CCR |= 1 << 0; // Enable The DMA To Work
   }

   // true, solange ein vorhergehender Sendevorgang nicht abgeschlossen ist.
   inline static bool isBusy() { return busy; }
};

// Verweis auf die (einzige) Instanz dieser Klasse.
extern UrsUsart1DmaClass& ursUsart1Dma;
#endif